@@ -22,11 +22,11 @@ import locale
 import contextlib
 import math
 from copy import copy
-from decimal import Decimal, getcontext
+from decimal import Decimal
 from itertools import product
 import urllib.parse
 
-from .exceptions import ElementPathValueError, XPATH_ERROR_CODES
+from .exceptions import ElementPathError, ElementPathValueError, XPATH_ERROR_CODES
 from .namespaces import XQT_ERRORS_NAMESPACE, XSD_NAMESPACE, XPATH_FUNCTIONS_NAMESPACE, \
     XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE
 from .xpath_nodes import AttributeNode, TextNode, NamespaceNode, TypedAttribute, \
@@ -40,8 +40,6 @@ from .schema_proxy import AbstractSchemaProxy
 from .tdop import Token, MultiLabel
 from .xpath_context import XPathSchemaContext
 
-getcontext().prec = 30
-
 UNICODE_CODEPOINT_COLLATION = "http://www.w3.org/2005/xpath-functions/collation/codepoint"
 
 XSD_SPECIAL_TYPES = {XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE}
@@ -96,7 +94,7 @@ class XPathToken(Token):
             return '$%s variable reference' % (self[0].value if self._items else '')
         elif symbol == ',':
             return 'comma operator' if self.parser.version > '1.0' else 'comma symbol'
-        elif label in ('axis', 'function', 'sequence type', 'kind test', 'constructor'):
+        elif label.endswith('function') or label in ('axis', 'sequence type', 'kind test'):
             return '%r %s' % (symbol, label)
         return super(XPathToken, self).__str__()
 
@@ -105,7 +103,7 @@ class XPathToken(Token):
         symbol, label = self.symbol, self.label
         if label == 'axis':
             return '%s::%s' % (self.symbol, self[0].source)
-        elif label in ('function', 'sequence type', 'kind test', 'constructor'):
+        elif label.endswith('function') or label in ('sequence type', 'kind test'):
             return '%s(%s)' % (self.symbol, ', '.join(item.source for item in self))
         elif symbol == ':':
             return '%s:%s' % (self[0].source, self[1].source)
@@ -139,7 +137,7 @@ class XPathToken(Token):
             return False
         elif self.symbol != ':':
             return True
-        return self[1].label not in ('function', 'constructor')
+        return not self[1].label.endswith('function')
 
     ###
     # Tokens tree analysis methods
@@ -202,12 +200,17 @@ class XPathToken(Token):
             for k, result in enumerate(selector(copy(context))):
                 if k == 0:
                     item = result
-                elif not self.parser.compatibility_mode:
+                elif self.parser.compatibility_mode:
+                    break
+                elif isinstance(context, XPathSchemaContext):
+                    # Multiple schema nodes are ignored but do not raise. The target
+                    # of schema context selection is XSD type association and multiple
+                    # nodes coherency is already checked at schema level.
+                    break
+                else:
                     raise self.wrong_context_type(
                         "a sequence of more than one item is not allowed as argument"
                     )
-                else:
-                    break
             else:
                 if item is None:
                     if not required:
@@ -227,7 +230,9 @@ class XPathToken(Token):
                 elif issubclass(cls, float) or issubclass(float, cls):
                     return self.number_value(item)
 
-            if self.parser.version > '1.0':
+            if self.parser.version == '1.0':
+                code = 'XPTY0004'
+            else:
                 value = self.data_value(item)
                 if isinstance(value, cls):
                     return value
@@ -240,8 +245,10 @@ class XPathToken(Token):
                     except (TypeError, ValueError):
                         pass
 
-            message = "the %s argument %r is not an instance of %r"
-            raise self.error('XPTY0004', message % (ordinal(index + 1), item, cls))
+                code = 'FOTY0012' if value is None else 'XPTY0004'
+
+            message = "the type of the {} argument is {!r} instead of {!r}"
+            raise self.error(code, message.format(ordinal(index + 1), type(item), cls))
 
         return item
 
@@ -283,15 +290,20 @@ class XPathToken(Token):
         except StopIteration:
             return
         else:
+            item = getattr(context, 'item', None)
+
             try:
                 next(selector)
             except StopIteration:
                 if isinstance(value, UntypedAtomic):
                     value = str(value)
-                if isinstance(context, XPathSchemaContext):
-                    return value
-                if self.xsd_types and isinstance(value, str):
-                    xsd_type = self.get_xsd_type(context.item)
+
+                if not isinstance(context, XPathSchemaContext) and \
+                        item is not None and \
+                        self.xsd_types and \
+                        isinstance(value, str):
+
+                    xsd_type = self.get_xsd_type(item)
                     if xsd_type is None or xsd_type.name in XSD_SPECIAL_TYPES:
                         pass
                     else:
@@ -300,6 +312,7 @@ class XPathToken(Token):
                         except (TypeError, ValueError):
                             msg = "Type {!r} is not appropriate for the context"
                             raise self.wrong_context_type(msg.format(type(value)))
+
                 return value
             else:
                 msg = "atomized operand is a sequence of length greater than one"
@@ -506,13 +519,13 @@ class XPathToken(Token):
             elif isinstance(self.label, MultiLabel):
                 self.label = 'function'
         elif namespace == XSD_NAMESPACE:
-            if self.label != 'constructor':
+            if self.label != 'constructor function':
                 msg = "a name, a wildcard or a constructor function expected"
                 raise self.wrong_syntax(msg, code='XPST0017')
             elif isinstance(self.label, MultiLabel):
-                self.label = 'constructor'
+                self.label = 'constructor function'
         else:
-            raise self.wrong_syntax("a name, a wildcard, a function or a constructor expected")
+            raise self.wrong_syntax("a name, a wildcard or a function expected")
 
         self.namespace = namespace
 
@@ -666,6 +679,8 @@ class XPathToken(Token):
             xsd_type = self.xsd_types.get(item)
         elif isinstance(item, AttributeNode):
             xsd_type = self.xsd_types.get(item[0])
+        elif isinstance(item, (TypedAttribute, TypedElement)):
+            return item.type
         else:
             xsd_type = self.xsd_types.get(item.tag)
 
@@ -700,6 +715,9 @@ class XPathToken(Token):
         :return: a TypedAttribute or a TypedElement, or the argument \
         if it's not matching any associated XSD type.
         """
+        if isinstance(item, (TypedAttribute, TypedElement)):
+            return item
+
         xsd_type = self.get_xsd_type(item)
         if not xsd_type:
             return item
@@ -871,13 +889,19 @@ class XPathToken(Token):
             return ''.join(etree_iter_strings(obj.getroot()))
         elif isinstance(obj, bool):
             return 'true' if obj else 'false'
-        elif isinstance(obj, (float, Decimal)):
+        elif isinstance(obj, Decimal):
+            value = format(obj, 'f')
+            if '.' in value:
+                return value.rstrip('0').rstrip('.')
+            return value
+
+        elif isinstance(obj, float):
             if math.isnan(obj):
                 return 'NaN'
             elif math.isinf(obj):
                 return str(obj).upper()
-            value = str(obj)
 
+            value = str(obj)
             if '.' in value:
                 value = value.rstrip('0').rstrip('.')
             if '+' in value:
@@ -910,30 +934,36 @@ class XPathToken(Token):
 
         return code  # returns an unprefixed code (without prefix the namespace is not checked)
 
-    def error(self, code, message=None):
+    def error(self, code, message_or_error=None):
         """
         Returns an XPath error instance related with a code. An XPath/XQuery/XSLT error code is an
         alphanumeric token starting with four uppercase letters and ending with four digits.
 
-        :param code: the error code.
-        :param message: an optional custom additional message.
+        :param code: the error code as QName or string.
+        :param message_or_error: an optional custom additional message.
         """
-        if ':' in code:
+        if isinstance(code, QName):
+            code, namespace = code.local_name, code.uri
+        elif ':' not in code:
+            namespace = None
+        else:
             try:
                 prefix, code = code.split(':')
             except ValueError:
                 raise ElementPathValueError(
-                    message='%r is not a QName' % code,
+                    message='%r is not a prefixed name' % code,
                     code=self.error_code('XPTY0004'),
                     token=self,
                 )
             else:
-                if self.parser.namespaces.get(prefix) != XQT_ERRORS_NAMESPACE:
-                    raise ElementPathValueError(
-                        message='%r namespace is required for error code' % XQT_ERRORS_NAMESPACE,
-                        code=self.error_code('XPTY0004'),
-                        token=self,
-                    )
+                namespace = self.parser.namespaces.get(prefix)
+
+        if namespace and namespace != XQT_ERRORS_NAMESPACE:
+            raise ElementPathValueError(
+                message='%r namespace is required' % XQT_ERRORS_NAMESPACE,
+                code=self.error_code('XPTY0004'),
+                token=self,
+            )
 
         try:
             error_class, default_message = XPATH_ERROR_CODES[code]
@@ -943,12 +973,17 @@ class XPathToken(Token):
                 code=self.error_code('XPTY0004'),
                 token=self,
             )
+
+        if message_or_error is None:
+            message = default_message
+        elif isinstance(message_or_error, str):
+            message = message_or_error
+        elif isinstance(message_or_error, ElementPathError):
+            message = message_or_error.message
         else:
-            return error_class(
-                message=message or default_message,
-                code=self.error_code(code),
-                token=self,
-            )
+            message = str(message_or_error)
+
+        return error_class(message, code=self.error_code(code), token=self)
 
     # Shortcuts for XPath errors, only the wrong_syntax
     def expected(self, *symbols, message=None, code='XPST0003'):
